﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

using Microsoft.WindowsAzure.Storage.Queue;

namespace CloudLibrary {
	public abstract class QueueListener {
		public string ConnectionName { get; set; }
		public string InputQueueName { get; set; }
		public string OutputQueueName { get; set; }
		public string BlobContainerName { get; set; }

		public async Task WatchQueueAsync(CancellationToken cancellationToken) {
			CloudQueueMessage queueMessage = null;
			var queue = QueueUtils.GetQueue(ConnectionName, InputQueueName);

			while (!cancellationToken.IsCancellationRequested) {
				queueMessage = QueueUtils.DequeueMessage(queue);
				if (queueMessage != null) {
					var queueMessageString = queueMessage.AsString;
					await QueueUtils.DeleteMessageAsync(queue, queueMessage);

					ProcessMessage(queueMessageString);

					// Done. So now put the incoming message string on the output queue (if there is one)
					if (OutputQueueName != null) {
						QueueUtils.EnqueueString(ConnectionName, OutputQueueName, queueMessageString);
					}
				}
				else {
					Thread.Sleep(2000);
				}
			}
		}

		protected abstract void ProcessMessage(string messageString);
	}
}
