using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.Storage;

using CloudLibrary;

namespace RoleFlipper {
	public class RoleFlipper : RoleEntryPoint {
		private readonly CancellationTokenSource cancellationTokenSource = new CancellationTokenSource();
		private readonly ManualResetEvent runCompleteEvent = new ManualResetEvent(false);

		private static string inputQueue = "dg-flipper-queue";
		private static string outputQueue = "dg-ready-queue";
		private static string connectionName = "DgStorageConnection";
		private static string blobContainerName = "dgdemoblobcontainer";
		private static QueueListener processor;

		public override void Run() {
			Trace.TraceInformation("RoleFlipper is running");

			try {
				this.RunAsync(this.cancellationTokenSource.Token).Wait();
			}
			finally {
				this.runCompleteEvent.Set();
			}
		}

		public override bool OnStart() {
			// Set the maximum number of concurrent connections
			ServicePointManager.DefaultConnectionLimit = 12;

			// For information on handling configuration changes
			// see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.

			bool result = base.OnStart();

			Trace.TraceInformation("RoleFlipper has been started");

			return result;
		}

		public override void OnStop() {
			Trace.TraceInformation("RoleFlipper is stopping");

			this.cancellationTokenSource.Cancel();
			this.runCompleteEvent.WaitOne();

			base.OnStop();

			Trace.TraceInformation("RoleFlipper has stopped");
		}

		private async Task RunAsync(CancellationToken cancellationToken) {
			processor = new FlipperProcessor();
			processor.ConnectionName = connectionName;
			processor.InputQueueName = inputQueue;
			processor.OutputQueueName = outputQueue;
			processor.BlobContainerName = blobContainerName;

			var queueTask = processor.WatchQueueAsync(cancellationTokenSource.Token);
		}
	}
}
