﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Blob;

namespace CloudLibrary {
	public static class BlobUtils {

		public static CloudBlobClient GetClient(string connectionName) {
			var blobClient = BlobUtils.InitialiseBlobStorage(connectionName);
			return blobClient;
		}

		public static CloudBlobClient InitialiseBlobStorage(string connectionName) {
			var storageAccount = CloudStorageAccount.Parse(CloudConfigurationManager.GetSetting(connectionName));
			var blobClient = storageAccount.CreateCloudBlobClient();
			return blobClient;
		}

		public static CloudBlobContainer EnsureContainer(string connectionName, string containerName) {
			var client = GetClient(connectionName);
			var blobContainer = EnsureContainer(client, containerName);
			return blobContainer;
		}

		public static CloudBlobContainer EnsureContainer(CloudBlobClient blobClient, string containerName) {
			var blobContainer = blobClient.GetContainerReference(containerName);
			// Create the blob if it doesn't already exist
			blobContainer.CreateIfNotExists();
			return blobContainer;
		}

		public static CloudBlockBlob UploadStreamToBlob(string connectionName, Stream stream, string containerName, string blobName) {
			var client = GetClient(connectionName);
			var container = EnsureContainer(client, containerName);
			CloudBlockBlob blockBlob = container.GetBlockBlobReference(blobName);
			stream.Position = 0;
			blockBlob.UploadFromStream(stream);
			return blockBlob;
		}

		public static void DownloadStreamFromBlob(string connectionName, Stream stream, string containerName, string blobName) {
			var client = GetClient(connectionName);
			var container = EnsureContainer(client, containerName);
			CloudBlockBlob blockBlob = container.GetBlockBlobReference(blobName);
			stream.Flush();
			stream.Position = 0;
			blockBlob.DownloadToStream(stream);
		}
	}
}
