﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Queue;

namespace CloudLibrary {
	public static class QueueUtils {

		public static CloudQueueClient GetClient(string connectionName) {
			var storageAccount = CloudStorageAccount.Parse(CloudConfigurationManager.GetSetting(connectionName));
			var queueClient = GetQueueClient(storageAccount);
			return queueClient;
		}

		public static CloudQueueClient GetQueueClient(CloudStorageAccount storageAccount) {
			var queueClient = storageAccount.CreateCloudQueueClient();
			return queueClient;
		}

		public static CloudQueue GetQueue(string connectionName, string queueName) {
			var client = GetClient(connectionName);
			var queue = GetQueue(client, queueName);
			return queue;
		}

		public static CloudQueue GetQueue(CloudQueueClient queueClient, string queueName) {
			var queue = queueClient.GetQueueReference(queueName);
			queue.CreateIfNotExists();
			return queue;
		}

		public static CloudQueueMessage EnqueueString(string connectionName, string queueName, string msgString) {
			var queue = GetQueue(connectionName, queueName);
			var queueMessage = EnqueueString(queue, msgString);
			return queueMessage;
		}

		public static CloudQueueMessage EnqueueString(CloudQueue queue, string msgString) {
			var message = new CloudQueueMessage(msgString);
			queue.AddMessage(message, new TimeSpan(8, 0, 0));
			return message;
		}

		public static CloudQueueMessage DequeueMessage(CloudQueue queue) {
			var invisibilityTime = new TimeSpan(0, 0, 30);
			var queuedMessage = queue.GetMessage(invisibilityTime);
			return queuedMessage;
		}

		public static Task DeleteMessageAsync(CloudQueue queue, CloudQueueMessage queueMessage) {
			return queue.DeleteMessageAsync(queueMessage);
		}
	}
}
